from torch.utils.data import Dataset
import torch.nn.functional as F
import torch
import torchaudio
import os
import random
from AudioUtils import stereo_tensor_to_mono, stretch_tensor

class AudioDataset(Dataset):
    audio_sample_rates = [11025]

    def __init__(self, input_dir):
        self.input_files = [
            os.path.join(root, f)
            for root, _, files in os.walk(input_dir)
            for f in files if f.endswith('.wav')
        ]

    def __len__(self):
        return len(self.input_files)

    def __getitem__(self, idx):
        # Load high-quality audio
        high_quality_path = self.input_files[idx]
        high_quality_audio, original_sample_rate = torchaudio.load(high_quality_path)
        high_quality_audio = stereo_tensor_to_mono(high_quality_audio)

        # Generate low-quality audio with random downsampling
        mangled_sample_rate = random.choice(self.audio_sample_rates)
        resample_low = torchaudio.transforms.Resample(original_sample_rate, mangled_sample_rate)
        low_quality_audio = resample_low(high_quality_audio)

        resample_high = torchaudio.transforms.Resample(mangled_sample_rate, original_sample_rate)
        low_quality_audio = resample_high(low_quality_audio)

        # Pad or truncate to match a fixed length
        target_length = 44100  # Adjust this based on your data
        high_quality_audio = self.pad_or_truncate(high_quality_audio, target_length)
        low_quality_audio = self.pad_or_truncate(low_quality_audio, target_length)

        return (high_quality_audio, original_sample_rate), (low_quality_audio, mangled_sample_rate)

    def pad_or_truncate(self, tensor, target_length):
        current_length = tensor.size(1)
        if current_length < target_length:
            # Pad with zeros
            padding = target_length - current_length
            tensor = F.pad(tensor, (0, padding))
        else:
            # Truncate to target length
            tensor = tensor[:, :target_length]
        return tensor
