import torch.nn as nn

def conv_residual_block(in_channels, out_channels, kernel_size=3, dilation=1):
    padding = (kernel_size // 2) * dilation
    return nn.Sequential(
        nn.Conv1d(in_channels, out_channels, kernel_size, dilation=dilation, padding=padding),
        nn.BatchNorm1d(out_channels),
        nn.PReLU(),
        nn.Conv1d(out_channels, out_channels, kernel_size, dilation=dilation, padding=padding),
        nn.BatchNorm1d(out_channels)
    )

class SISUGenerator(nn.Module):
    def __init__(self):
        super(SISUGenerator, self).__init__()
        layers = 4
        self.conv1 = nn.Sequential(
            nn.Conv1d(1, layers, kernel_size=7, padding=3),
            nn.BatchNorm1d(layers),
            nn.PReLU()
        )

        self.conv_blocks = nn.Sequential(
            conv_residual_block(layers, layers, kernel_size=3, dilation=1),
            conv_residual_block(layers, layers * 2, kernel_size=5, dilation=2),
            conv_residual_block(layers * 2, layers * 4, kernel_size=3, dilation=16),
            conv_residual_block(layers * 4, layers * 2, kernel_size=5, dilation=8),
            conv_residual_block(layers * 2, layers, kernel_size=5, dilation=2),
            conv_residual_block(layers, layers, kernel_size=3, dilation=1)
        )

        self.final_layer = nn.Sequential(
            nn.Conv1d(layers, 1, kernel_size=3, padding=1)
        )

    def forward(self, x):
        residual = x
        x = self.conv1(x)
        x = self.conv_blocks(x) + x  # Adding residual connection after blocks
        x = self.final_layer(x)
        return x + residual
